(load "projet.scm")
(define bloc-taille '?)
(define base 256) 
(define nb-digit
  (inexact->exact (/ (log base) (log 2))))

;**************** EQUILIBRAGE DU TEXTE *****************
;**************** COUPAGE   DU   TEXTE *****************
(define (calibre string) 
  (let ((reste (- bloc-taille (modulo (string-length string) bloc-taille))))
  (if (=  reste bloc-taille) string
      (string-append string (make-string reste #\space)))))

;; en O(longeur-du-texte/bloc-taille)
(define (decoupage texte)
  (letrec ((S (calibre texte))
           (l (string-length S))
           (nb-bloc (quotient l bloc-taille))
           (AUX (lambda(i k res)
                  (printf ".")
                  (if (= k 0)
                    (begin (printf "[1] DECOUPAGE DU TEXTE TERMINE ~n") res)
                    (AUX (+ i bloc-taille) 
                         (sub1 K) 
                         (append res (list(substring S i (+ i bloc-taille)))))))))
    (begin
      (printf "#################################################### ~n")
      (printf "TAILLE INITIALE DU TEXTE = ~a ~n" (string-length texte))
      (printf "TAILLE DU TEXTE CALIBRE = ~a ~n" l)
      (printf "NOMBRE DE BLOCS = ~a ~n" nb-bloc)
      (printf "TAILLE DU BLOC = ~a ~n" bloc-taille)
      (AUX 0 nb-bloc '()))))

;**************** CODAGE *****************

;; string->number (c(n-1),c(n-2),.......c(1)) =
;; [c(n-1)]*(base)^n-1 + [c(n-2)]*(base)^n-2 + ..... + c1 ou n=bloc-taille
;; Le nombre M obtenu via cette fonction vaux au plus MAX=(base^n - 1)=(256^bloc-taille)-1  
(define (string->number s)
  (printf "."  ) 
  (let ((L (string-length s)) (resultat 0) )
    (do ((i 0 (add1 i)))
      ((= i L) resultat)
      (set! resultat 
            (+ (* (char->integer (string-ref s i)) (expt 2 (* (- (sub1 L) i) nb-digit))) resultat)))))

(define (encrypte-string S cle-pub)
  (encrypte  (string->number S) cle-pub))

(define (CODAGE message cle-pub)
  (let ((n (cle-base cle-pub))) 
    (set! bloc-taille  ;; la taille du bloc change en fonction de n!! 
          (inexact->exact(floor (/ (log (+ 1 n)) (log base)))))
    (begin0
      (map (lambda(x)(encrypte-string x cle-pub)) (decoupage message))
      (printf "[2] CODAGE TERMINE ~n"))))
           
(define (lire-fichier nom)
  (define (loop flux)
    (let ((ligne (read-line flux)))
      (if (eof-object? ligne)
          ""
          (string-append  ligne (loop flux)))))
  (call-with-input-file nom loop))

(define (CODAGE-FICHIER nom cle-pub)
  (CODAGE (lire-fichier nom) cle-pub))

;**************** DECODAGE *****************

(define (number->string int)
  (define (aux int res)
    (if (= 0 int)
        res
        (aux (quotient int base) (cons (integer->char (modulo int base)) res))))
  (begin0 (list->string (aux int '()))
          (printf ".")))

(define (decrypte-number int cle-priv)
  (number->string (decrypte int cle-priv)))

(define (DECODAGE list-de-nombre cle-priv)
  (begin0
    (apply string-append (map (lambda(x) (decrypte-number x cle-priv)) list-de-nombre)) 
    (printf"[3]DECOGAE TERMINE")))

;;;;;;;;;;;;; test ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define **KEYS** (GENERATEUR-CLES-PUBLICS-PRIVEES)) 
(define KEY1 (extraire-CLE1-RSA **KEYS**))
(define KEY2 (extraire-CLE2-RSA **KEYS**)) 

;; p1 = cle1
;; p2 = cle2
;; De plus le modulo n est  129 chiffres : c'est le fameux RSA 129.
(define p1
'(114381625757888867669235779976146612010218296721242362562561842935706935245733897830597123563958705058989075147599290026879543541 . 745580037409))
(define p2
'(114381625757888867669235779976146612010218296721242362562561842935706935245733897830597123563958705058989075147599290026879543541 . 93173337432348835437044144231184828740466786227126166633332363868171459867939031261332999179163598955710515267841256382242701153))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(time (DECODAGE (CODAGE-FICHIER "Rapport-Projet-Scheme.txt" KEY1) KEY2))
