(define taille 20)
(define nb-test-fermat 3) 

(define (mulmod a b m)
  (remainder (* a b) m))

(define (sqrmod a m)
  (mulmod a a m))

(define (addmod a b m)
  (remainder (+ a b) m))

(define (submod a b m)
  (remainder (- a b) m))

;;;;;;;;;;;;;;; RANDOM ;;;;;;;;;;;;;;;;;;;;
; Cette fonction retourne un entier appartenant  [a,b]
(define ($random a b)
  (let ((dist (- b a))
        (T_max (expt 10 308)))
    (+ (random (+ (if ( < dist T_max)
                      dist T_max )
                  1))
       a)))

; Cette fonction retourne un nombre
; aleatoire < n et qui est premier avec n
(define (random-base n)
  (letrec ((aleatoire (random
                       (if (< (nb-chiffre n) 10)
                           n
                           (inexact->exact(floor (expt n (/ 1 5)))))))
           (iter (lambda(x)
                   (cond  ((= x 1) (random-base n)) 
                          ((= (gcd x n) 1) x)
                          (else(iter (+ x 1)))))))
    (iter aleatoire)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Cette fonction calcule a^n modulo m
(define (expmod a n m)
  (cond ((zero? n) 1)
	((even? n) (expmod (sqrmod a m) (/ n 2) m))
	(else (mulmod a (expmod a (- n 1) m) m))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (bezout a b f)
  (if (zero? b)
      (f a 1 0)
      (bezout b (modulo a b) (lambda (d u v) (f d v (- u (* (quotient a b) v)))))))

(define (invmod a n)
  (let ((res (bezout a n (lambda (d u v) (cons d u)))))
    (if (= 1 (car res))
        (modulo (cdr res) n)
        #f)))

;;;;;;;;;;;;;;   FERMAT ;;;;;;;;;;;;;;;;;;;


(define (OK? n)
  (define (fermat-test n)
    (let ((reste (modulo n 10)))
      (case reste
        ((1 3 7 9) (let ((a (random-base n))) (= (expmod a n n) a)))
        (else  #f))))      
  
  (define (OK_fermat? n times)
    (cond ((zero? times) true)
          ((fermat-test n) (OK_fermat? n (- times 1)))
          (else  false)))
  
  (and  (fermat-test n) (OK_fermat? n nb-test-fermat)))


;;;;;;;;;;;;;;;;;;;-------------------------------------------;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;GENERATEUR DE NOMBRES PREMEIRS A K chiffres;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;-------------------------------------------;;;;;;;;;;;;;;;;;;;
(define (nb-chiffre r)
  (let ((N (abs r)))
    (inexact->exact (floor (add1 (/ (log N) (log 10)))))))

; Cette fonction se tente de trouver un nombre premier qui a nb_chiffres 
(define (trouve-premier nb_chiffre)
  (printf "Recherche d'un nombre premier  ~a chiffres,veuillez patienter ... " nb_chiffre )  
  (let* ((debut (expt 10 (- nb_chiffre 1)))
         (fin (sub1 (expt 10 nb_chiffre)))
         (x (choisir_premier_entre* debut fin)))
    (display (= nb_chiffre (nb-chiffre x)))
    (newline)
    x))

;; Le theorme de Lavallee Poussin donne la rarefaction des
;; nombres premiers : la probabilite pour que
;; (p tire aleatoirement soit premier) equivaut  1/log(p).
(define (choisir_premier_entre* min max)                  
  (let* ((MI (if (even? min) (add1 min) min))
         (SI (if (even? max) (sub1 max) max)) 
         (x ($random MI SI))
         (essai (if (even? x) (add1 x) x)))
    
    (define (jump n)
      (if (= (modulo n 10) 3)(+ n 4)(+ n 2)))
    
    (define (catch n TOURS) 
      (cond ((OK? n)  n)
            ((and (or (>= n essai)(>= (jump n) SI)) (= TOURS 2)) 
             (printf "Cycle-trouve: pas de nombres premiers dans l intervalle [~a , ~a]" min max))
            ((> (jump n) SI) (catch MI 2)) 
            (else (catch (jump n) TOURS))))
              
    (catch essai 1)))


;;;;;;;;;;;;;;;;;;;--------;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;   RSA  ;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;--------;;;;;;;;;;;;;;;;;;;
(define genere-cle cons)
(define cle-base car)
(define cle-exposant cdr)

(define (GENERATEUR-CLES-PUBLICS-PRIVEES) 
  (display "GENERATION DES CLES PUBLICS ET PRIVEE")
  (newline)
  (let* ((t taille)
         (p (trouve-premier taille))
         (q (trouve-premier taille))
         (n (* p q))
         (phi (* (sub1 p) (sub1 q)))
         (e (random-base phi))
         (d (invmod e phi)))
    (printf " p:= ~a ~n q:= ~a ~n n:= ~a ~n" p q n)
    (genere-cle (genere-cle n e) (genere-cle n d))))

;######################################################
;####################### TESTS ########################

(define (extraire-CLE1-RSA RSA-CLES)
  (car RSA-CLES))

(define (extraire-CLE2-RSA RSA-CLES)
  (cdr RSA-CLES))

(define (encrypte nb cle1) ;; cle1 est la cle public
  (expmod nb (cle-exposant cle1) (cle-base cle1)))

(define (decrypte nb cle2) ;; cle2 est la cle prive
  (expmod nb (cle-exposant cle2) (cle-base cle2)))
